/*******************************************************************************
 * Copyright (C) Altimetrik 2018. All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Altimetrik. You shall not disclose such Confidential Information
 * and shall use it only in accordance with the terms and conditions
 * entered into with Altimetrik.
 ******************************************************************************/
package com.altimetrik.playground.innovation.controller;

import java.net.URI;
import java.net.URISyntaxException;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.altimetrik.playground.innovation.bean.BaseResponse;
import com.altimetrik.playground.innovation.bean.PgIdeaArtifact;
import com.altimetrik.playground.innovation.bean.PgIdeaArtifactListResponse;
import com.altimetrik.playground.innovation.bean.PgIdeaRequest;
import com.altimetrik.playground.innovation.bean.PgIdeaResponse;
import com.altimetrik.playground.innovation.bean.PgIdeaSearchRequest;
import com.altimetrik.playground.innovation.bean.PgIdeaSearchResponse;
import com.altimetrik.playground.innovation.bean.PgIdeaTeamInviteRequest;
import com.altimetrik.playground.innovation.bean.PgIdeaTeamListResponse;
import com.altimetrik.playground.innovation.exception.PgApplicationException;
import com.altimetrik.playground.innovation.service.PgIdeaService;

@RestController
public class PgIdeaController extends BaseController {

	@Autowired
	private PgIdeaService pgIdeaService;

	@GetMapping(value = "/v1/ideas")
	public ResponseEntity<PgIdeaSearchResponse> listIdeas(@RequestParam(value = "search", required = false) String search, @RequestParam("mine") Boolean mine,
			@RequestParam(value = "offset", required = false) Integer offset, @RequestParam(value = "limit", required = false) Integer limit,
			@RequestParam(value = "ids", required = false) String[] ids) throws PgApplicationException {
		PgIdeaSearchRequest req = new PgIdeaSearchRequest();
		if (search != null)
			req.setSearch(search);
		if (mine != null)
			req.setMine(mine);
		if (offset != null)
			req.setOffset(offset);
		if (limit != null)
			req.setLimit(limit);
		if (ids != null)
			req.setIds(ids);
		return new ResponseEntity<>(pgIdeaService.getAllIdeas(req), HttpStatus.OK);
	}

	@GetMapping(value = "/v1/ideas/{ideaId}")
	public ResponseEntity<PgIdeaResponse> getIdea(@PathVariable("ideaId") final String ideaId) throws PgApplicationException {
		return new ResponseEntity<>(pgIdeaService.getIdea(ideaId), HttpStatus.OK);
	}

	@PostMapping(value = "/v1/ideas")
	public ResponseEntity<PgIdeaResponse> createIdea(@Validated @RequestBody PgIdeaRequest pgIdeaRequest) throws PgApplicationException, URISyntaxException {
		PgIdeaResponse createIdea = pgIdeaService.createIdea(pgIdeaRequest);
		return ResponseEntity.created(new URI("/v1/ideas/" + createIdea.getIdeaId())).body(createIdea);
	}

	@PatchMapping(value = "/v1/ideas/{ideaId}")
	public ResponseEntity<PgIdeaResponse> updateIdea(@PathVariable("ideaId") final String ideaId, @RequestBody PgIdeaRequest pgIdeaRequest) throws PgApplicationException {
		return ResponseEntity.ok(pgIdeaService.updateIdea(ideaId, pgIdeaRequest));
	}

	@GetMapping(value = "/v1/ideas/{ideaId}/artifacts")
	public ResponseEntity<PgIdeaArtifactListResponse> getIdeaArtifacts(@PathVariable("ideaId") final String ideaId) throws PgApplicationException {
		return new ResponseEntity<>(pgIdeaService.getArtifact(ideaId), HttpStatus.OK);
	}

	@PostMapping(value = "/v1/ideas/{ideaId}/artifacts")
	public ResponseEntity<PgIdeaArtifactListResponse> addArtifact(@PathVariable("ideaId") final String ideaId, @Validated @RequestBody PgIdeaArtifact req)
			throws PgApplicationException {
		return new ResponseEntity<>(pgIdeaService.addArtifact(ideaId, req), HttpStatus.OK);
	}

	@GetMapping(value = "/v1/ideas/{ideaId}/team")
	public ResponseEntity<PgIdeaTeamListResponse> ideaTeamList(@PathVariable("ideaId") final String ideaId) throws PgApplicationException {
		return new ResponseEntity<>(pgIdeaService.getTeamList(ideaId), HttpStatus.OK);
	}

	@GetMapping(value = "/v1/ideas/{ideaId}/team/requests")
	public ResponseEntity<PgIdeaTeamListResponse> ideaTeamRequestList(@PathVariable("ideaId") final String ideaId) throws PgApplicationException {
		return new ResponseEntity<>(pgIdeaService.listTeamRequests(ideaId), HttpStatus.OK);
	}

	@PostMapping(value = "/v1/ideas/{ideaId}/team/requests/invite")
	public ResponseEntity<BaseResponse> ideaInviteRequest(@PathVariable("ideaId") final String ideaId, @Validated @RequestBody PgIdeaTeamInviteRequest request)
			throws PgApplicationException {
		return new ResponseEntity<>(pgIdeaService.inviteTeamRequests(ideaId, request), HttpStatus.OK);
	}

	@PostMapping(value = "/v1/ideas/{ideaId}/team/invite/{requestId}/reject")
	public ResponseEntity<BaseResponse> ideaRejectInvite(@PathVariable("ideaId") final String ideaId, @PathVariable("requestId") final String requestId)
			throws PgApplicationException {
		return new ResponseEntity<>(pgIdeaService.rejectTeamRequest(ideaId, requestId), HttpStatus.OK);
	}

	@PostMapping(value = "/v1/ideas/{ideaId}/team/invite/{requestId}/accept")
	public ResponseEntity<BaseResponse> ideaAcceptInvite(@PathVariable("ideaId") final String ideaId, @PathVariable("requestId") final String requestId)
			throws PgApplicationException {
		return new ResponseEntity<>(pgIdeaService.acceptTeamRequest(ideaId, requestId), HttpStatus.OK);
	}

	// @RequestMapping(value = "/v1/ideas/{ideaId}/status", method =
	// RequestMethod.POST)
	// public ResponseEntity<BaseResponse>
	// updateIdeaStatus(@PathVariable("ideaId")
	// final long ideaId, @RequestBody PgIdeaStatusChangeRequest pgIdeaRequest)
	// {
	// return new ResponseEntity<BaseResponse>(new BaseResponse(),
	// HttpStatus.OK);
	// }
	//
	// @RequestMapping(value = "/v1/ideas/{ideaId}/status", method =
	// RequestMethod.GET)
	// public ResponseEntity<PgIdeaStatusChangeRequest>
	// getIdeaStatus(@PathVariable("ideaId") final long ideaId) {
	// return new ResponseEntity<PgIdeaStatusChangeRequest>(new
	// PgIdeaStatusChangeRequest(), HttpStatus.OK);
	// }

	// @RequestMapping(value = "/v1/ideas/{ideaId}/share", method =
	// RequestMethod.POST)
	// public ResponseEntity<BaseResponse> shareIdea(@RequestBody
	// PgIdeaShareRequest
	// pgIdeaShareRequest) {
	// return new ResponseEntity<BaseResponse>(new BaseResponse(),
	// HttpStatus.OK);
	// }

	@PostMapping(value = "/v1/ideas/{ideaId}/team/requests/join")
	public ResponseEntity<BaseResponse> ideaJoinRequest(@PathVariable("ideaId") final String ideaId) throws PgApplicationException {
		return ResponseEntity.ok(pgIdeaService.joinRequest(ideaId));
	}

	@PostMapping(value = "/v1/ideas/{ideaId}/team/join/{requestId}/accept")
	public ResponseEntity<BaseResponse> ideaAcceptJoin(@PathVariable("ideaId") final String ideaId, @PathVariable("requestId") final String requestId)
			throws PgApplicationException {
		return ResponseEntity.ok(pgIdeaService.acceptJoinRequests(ideaId, requestId));
	}

	@PostMapping(value = "/v1/ideas/{ideaId}/team/join/{requestId}/reject")
	public ResponseEntity<BaseResponse> ideaRejectJoin(@PathVariable("ideaId") final String ideaId, @PathVariable("requestId") final String requestId)
			throws PgApplicationException {
		return ResponseEntity.ok(pgIdeaService.rejectJoinRequests(ideaId, requestId));
	}

	// @RequestMapping(value = "/v1/ideas/artifacts/file/{fileName:.+}", method
	// =
	// RequestMethod.GET)
	// public ResponseEntity<Resource> download(@PathVariable final String
	// fileName,
	// HttpServletRequest request) throws MalformedURLException {
	// final Resource resource = new UrlResource("");
	// ResponseEntity<Resource> response =
	// ResponseEntity.ok().contentType(MediaType.parseMediaType("application/octet-stream"))
	// .header(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=\"" +
	// resource.getFilename() + "\"").body(resource);
	// return response;
	// }

	// @RequestMapping(value = "/v1/ideas/status/history/{ideaId}", method =
	// RequestMethod.GET)
	// public ResponseEntity<PgIdeaStatusHistoryListResponse>
	// getIdeaStatusHistory(@PathVariable("ideaId") final Long ideaId) {
	// return new ResponseEntity<PgIdeaStatusHistoryListResponse>(new
	// PgIdeaStatusHistoryListResponse(), HttpStatus.OK);
	// }

}
