/*******************************************************************************
 * Copyright (C) Altimetrik 2018. All rights reserved.
 * 
 * This software is the confidential and proprietary information of Altimetrik.
 * You shall not disclose such Confidential Information and shall use it only in
 * accordance with the terms and conditions entered into with Altimetrik.
 ******************************************************************************/

package com.altimetrik.playground.innovation.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.Random;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Component;

import com.altimetrik.playground.innovation.bean.BaseResponse;
import com.altimetrik.playground.innovation.bean.PgIdeaArtifact;
import com.altimetrik.playground.innovation.bean.PgIdeaArtifactListResponse;
import com.altimetrik.playground.innovation.bean.PgIdeaRequest;
import com.altimetrik.playground.innovation.bean.PgIdeaResponse;
import com.altimetrik.playground.innovation.bean.PgIdeaSearchRequest;
import com.altimetrik.playground.innovation.bean.PgIdeaSearchResponse;
import com.altimetrik.playground.innovation.bean.PgIdeaTeamInviteRequest;
import com.altimetrik.playground.innovation.bean.PgIdeaTeamListResponse;
import com.altimetrik.playground.innovation.bean.PgIdeaTeamResponse;
import com.altimetrik.playground.innovation.bean.StatusDetails;
import com.altimetrik.playground.innovation.constants.ArtifactTypeEnum;
import com.altimetrik.playground.innovation.constants.IdeaStatusEnum;
import com.altimetrik.playground.innovation.constants.IdeaTypeEnum;
import com.altimetrik.playground.innovation.constants.ReviewLevelEnum;
import com.altimetrik.playground.innovation.constants.RoleTypeEnum;
import com.altimetrik.playground.innovation.constants.TeamMemberRequestModeEnum;
import com.altimetrik.playground.innovation.constants.TeamMemberRequestStatusEnum;
import com.altimetrik.playground.innovation.entity.PgCategoryEntity;
import com.altimetrik.playground.innovation.entity.PgIdeaArtifactEntity;
import com.altimetrik.playground.innovation.entity.PgIdeaEntity;
import com.altimetrik.playground.innovation.entity.PgReviewersEntity;
import com.altimetrik.playground.innovation.entity.PgTeamEntity;
import com.altimetrik.playground.innovation.exception.InnovationServiceException;
import com.altimetrik.playground.innovation.exception.PgApplicationException;
import com.altimetrik.playground.innovation.repository.PgIdeaCategoryRepository;
import com.altimetrik.playground.innovation.repository.PgIdeaRepository;
import com.altimetrik.playground.innovation.service.PgIdeaService;
import com.altimetrik.playground.innovation.util.UserUtil;

import lombok.extern.slf4j.Slf4j;

@Component
@Slf4j
public class PgIdeaServiceImpl implements PgIdeaService {

	private static final String IDEA_WITH_ID = "Idea with ID (";
	private static final String NOT_FOUND = "not.found";
	private static final String USER_NOT_AUTHORIZED = "user.not.authorized";
	private static final String USER_NOT_AUTHORIZED_MSG = "Current user not authorized to perform this action on this request.";

	@Autowired
	private PgIdeaRepository pgIdeaRepository;

	@Autowired
	private PgIdeaCategoryRepository pgIdeaCategoryRepository;

	@Autowired
	private MongoTemplate mongoTemplate;

	@Autowired
	private UserUtil userUtil;

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.altimetrik.playground.innovation.service.PgIdeaService#getIdea(java.
	 * lang.String)
	 */
	@Override
	public PgIdeaResponse getIdea(String ideaId) throws PgApplicationException {
		PgIdeaEntity ideaEntity = getIdeaEntity(ideaId);
		try {
			validateUserAccessToIdea(ideaEntity);
		}
		catch (InnovationServiceException e) {
			if (!ideaEntity.getPublished() || ideaEntity.getIdeaType().equals(IdeaTypeEnum.PRIVATE))
				throw e;
		}
		PgIdeaResponse response = prepareIdeaResponse(ideaEntity);
		response.setStatus(new StatusDetails(StatusDetails.SUCCESS));
		return response;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.altimetrik.playground.innovation.service.PgIdeaService#getAllIdeas(
	 * com.altimetrik.playground.innovation.bean.PgIdeaSearchRequest)
	 */
	@Override
	public PgIdeaSearchResponse getAllIdeas(PgIdeaSearchRequest req) throws PgApplicationException {
		PgIdeaSearchResponse response = new PgIdeaSearchResponse();
		List<PgIdeaResponse> result = new ArrayList<>();
		List<Criteria> andCriterias = new ArrayList<>();
		List<Criteria> orCriterias = new ArrayList<>();
		Criteria criteria;
		if (StringUtils.isNotBlank(req.getSearch())) {
			Criteria title = Criteria.where("title").regex(req.getSearch().trim(), "i");
			Criteria desc = Criteria.where("description").regex(req.getSearch().trim(), "i");
			andCriterias.add(new Criteria().orOperator(title, desc));
		}
		if (req.getIds() != null) {
			andCriterias.add(Criteria.where("_id").in(req.getIds()));
		}
		if (req.getMine()) {
			andCriterias.add(Criteria.where("team").elemMatch(Criteria.where("userId").is(userUtil.getLoggedInUser()).andOperator(new Criteria()
					.orOperator(Criteria.where("role").is(RoleTypeEnum.INNOVATOR), Criteria.where("role").is(RoleTypeEnum.CO_INNOVATOR),
							Criteria.where("role").is(RoleTypeEnum.MEMBER))
					.andOperator(
							new Criteria().orOperator(Criteria.where("requestStatus").is(TeamMemberRequestStatusEnum.ACCEPTED), Criteria.where("requestStatus").exists(false))))));
		}
		else {
			andCriterias.add(Criteria.where("ideaType").is(IdeaTypeEnum.PUBLIC));
			andCriterias.add(Criteria.where("published").is(true));
			andCriterias.add(Criteria.where("team").not().elemMatch(Criteria.where("userId").is(userUtil.getLoggedInUser())));
		}
		if (andCriterias.isEmpty() && orCriterias.isEmpty()) {
			criteria = new Criteria();
		}
		else if (andCriterias.isEmpty() && !orCriterias.isEmpty()) {
			criteria = new Criteria().orOperator(orCriterias.toArray(new Criteria[orCriterias.size()]));
		}
		else if (!andCriterias.isEmpty() && orCriterias.isEmpty()) {
			criteria = new Criteria().andOperator(andCriterias.toArray(new Criteria[andCriterias.size()]));
		}
		else {
			criteria = new Criteria().andOperator(andCriterias.toArray(new Criteria[andCriterias.size()])).orOperator(orCriterias.toArray(new Criteria[orCriterias.size()]));
		}
		Query query = new Query(criteria);
		if (req.getLimit() != null)
			query.limit(req.getLimit());
		if (req.getOffset() != null)
			query.skip(req.getOffset());
		List<PgIdeaEntity> entityList = mongoTemplate.find(query, PgIdeaEntity.class);
		entityList.forEach(e -> {
			PgIdeaResponse ideaResponse = prepareIdeaResponse(e);
			result.add(ideaResponse);
		});
		response.setResult(result);

		response.setStatus(new StatusDetails(StatusDetails.SUCCESS));
		return response;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.altimetrik.playground.innovation.service.PgIdeaService#addArtifact(
	 * java.lang.String, com.altimetrik.playground.innovation.bean.PgIdeaArtifact)
	 */
	@Override
	public PgIdeaArtifactListResponse addArtifact(String ideaId, PgIdeaArtifact req) throws PgApplicationException {
		PgIdeaEntity ideaEntity = getIdeaEntity(ideaId);
		validateUserAccessToIdea(ideaEntity);
		List<PgIdeaArtifactEntity> artifactList = ideaEntity.getArtifacts();
		if (artifactList == null)
			artifactList = new ArrayList<>();
		PgIdeaArtifactEntity artifact = new PgIdeaArtifactEntity(req.getLinkName(), req.getLinkUrl(), ArtifactTypeEnum.LINK, req.getFileName());
		artifact.setCreatedBy(userUtil.getLoggedInUser());
		artifact.setCreatedDate(new Date());
		artifactList.add(artifact);
		ideaEntity.setArtifacts(artifactList);
		ideaEntity = pgIdeaRepository.save(ideaEntity);

		return prepareArtifactResponse(ideaEntity);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.altimetrik.playground.innovation.service.PgIdeaService#getArtifact(
	 * java.lang.String)
	 */
	@Override
	public PgIdeaArtifactListResponse getArtifact(String ideaId) throws PgApplicationException {
		PgIdeaEntity ideaEntity = getIdeaEntity(ideaId);
		validateUserAccessToIdea(ideaEntity);
		return prepareArtifactResponse(ideaEntity);
	}

	/**
	 * @param ideaEntity
	 * @return
	 */
	private PgIdeaArtifactListResponse prepareArtifactResponse(PgIdeaEntity ideaEntity) {
		PgIdeaArtifactListResponse resp = new PgIdeaArtifactListResponse();
		List<PgIdeaArtifact> result = new ArrayList<>();
		resp.setIdeaId(ideaEntity.getId());
		if (ideaEntity.getArtifacts() != null) {
			ideaEntity.getArtifacts().forEach(entity -> result.add(new PgIdeaArtifact(entity.getId(), entity.getLinkName(), entity.getType(), entity.getLinkUrl(),
					entity.getFileName(), entity.getCreatedBy(), entity.getCreatedDate())));
		}
		resp.setResult(result);
		resp.setStatus(new StatusDetails(StatusDetails.SUCCESS));
		return resp;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.altimetrik.playground.innovation.service.PgIdeaService#createIdea(com
	 * .altimetrik.playground.innovation.bean.PgIdeaRequest)
	 */
	@Override
	public PgIdeaResponse createIdea(PgIdeaRequest pgIdeaRequest) throws PgApplicationException {

		innitialCheckForIdeaCreation(pgIdeaRequest);
		String loggedInUser = userUtil.getLoggedInUser();
		PgIdeaEntity entity = new PgIdeaEntity();
		entity.setTitle(pgIdeaRequest.getTitle());
		entity.setDescription(pgIdeaRequest.getDescription());
		entity.setBusinessRelevance(pgIdeaRequest.getBusinessRelevance());
		entity.setCompetenceCenter(pgIdeaRequest.getCompetenceCenter() == null ? Boolean.FALSE : pgIdeaRequest.getCompetenceCenter());
		entity.setIdeaType(pgIdeaRequest.getIdeaType() == null ? IdeaTypeEnum.PUBLIC : pgIdeaRequest.getIdeaType());
		entity.setStatus(pgIdeaRequest.getStatus() == null ? IdeaStatusEnum.NOT_SUBMITTED : pgIdeaRequest.getStatus());
		entity.setPublished(pgIdeaRequest.getPublished() == null ? Boolean.FALSE : pgIdeaRequest.getPublished());
		entity.setDisclosed(pgIdeaRequest.getDisclosed() == null ? Boolean.FALSE : pgIdeaRequest.getDisclosed());
		entity.setLocked(pgIdeaRequest.getLocked() == null ? Boolean.FALSE : pgIdeaRequest.getLocked());
		entity.setBusinessDomain(pgIdeaRequest.getBusinessDomain());
		entity.setTechnologyDomain(pgIdeaRequest.getTechnologyDomain());
		entity.setTechStack(pgIdeaRequest.getTechStack());
		entity.setNovelty(pgIdeaRequest.getNovelty());
		entity.setClaims(pgIdeaRequest.getClaims());
		entity.setCompleteness(pgIdeaRequest.getCompleteness());
		entity.setAccessLevel(pgIdeaRequest.getAccessLevel());
		if (pgIdeaRequest.getIdeaCategoryId() != null) {
			entity.setIdeaCategoryId(pgIdeaRequest.getIdeaCategoryId());
			entity.setL0ReviewerId(getL0Reviwer(pgIdeaRequest.getIdeaCategoryId()));
		}
		// Setting the creator for an idea
		PgTeamEntity creator = new PgTeamEntity();
		creator.setRole(RoleTypeEnum.INNOVATOR);
		log.info("Creator for idea : {}", loggedInUser);
		creator.setUserId(loggedInUser);
		creator.setRequestStatus(TeamMemberRequestStatusEnum.ACCEPTED);
		creator.setCreatedBy(loggedInUser);
		creator.setCreatedDate(new Date());
		List<PgTeamEntity> team = new ArrayList<>();
		team.add(creator);
		entity.setTeam(team);
		entity.setCreatedBy(loggedInUser);
		entity.setCreatedDate(new Date());
		entity.setLastModifiedBy(loggedInUser);
		entity = pgIdeaRepository.insert(entity);
		PgIdeaResponse response = prepareIdeaResponse(entity);
		response.setStatus(new StatusDetails(StatusDetails.SUCCESS));
		return response;
	}

	/**
	 * @param pgIdeaRequest
	 * @throws InnovationServiceException
	 */
	private void innitialCheckForIdeaCreation(PgIdeaRequest pgIdeaRequest) throws InnovationServiceException {
		if (pgIdeaRequest.getStatus() != null && pgIdeaRequest.getStatus().equals(IdeaStatusEnum.APPROVED))
			throw new InnovationServiceException("invalid", "Idea cannot be created as an approved idea");
		if (StringUtils.isBlank(pgIdeaRequest.getTitle()))
			throw new InnovationServiceException("idea.not.found", "Idea Title Not Found");
		else {
			Optional<PgIdeaEntity> findByTitle = pgIdeaRepository.findByTitle(pgIdeaRequest.getTitle());
			if (findByTitle.isPresent())
				throw new InnovationServiceException("idea.exist", "Idea with given title already exist.");
		}
		if (pgIdeaRequest.getStatus() != null && pgIdeaRequest.getStatus().equals(IdeaStatusEnum.SUBMITTED)) {
			if (pgIdeaRequest.getPublished() == null || !pgIdeaRequest.getPublished())
				throw new InnovationServiceException("not.published", "Idea Not Published for Submission");
			if (pgIdeaRequest.getIdeaCategoryId() != null)
				pgIdeaCategoryRepository.findById(pgIdeaRequest.getIdeaCategoryId())
						.orElseThrow(() -> new InnovationServiceException("category.not.found", "Category Not Found with ID : " + pgIdeaRequest.getIdeaCategoryId()));
			else
				throw new InnovationServiceException(NOT_FOUND, "An idea has to part of a category before Submission.");
			if (StringUtils.isNotBlank(pgIdeaRequest.getNovelty()))
				throw new InnovationServiceException(NOT_FOUND, "An idea has to contain novelty before Submission.");
			if (StringUtils.isNotBlank(pgIdeaRequest.getClaims()))
				throw new InnovationServiceException(NOT_FOUND, "An idea has to contain claims before Submission.");
			if (StringUtils.isNotBlank(pgIdeaRequest.getCompleteness()))
				throw new InnovationServiceException(NOT_FOUND, "An idea has to contain completeness before Submission.");
		}
	}

	/**
	 * to create the response model.
	 * @param entity
	 * @return
	 */
	private PgIdeaResponse prepareIdeaResponse(PgIdeaEntity entity) {
		PgIdeaResponse response = new PgIdeaResponse();
		response.setIdeaId(entity.getId());
		response.setTitle(entity.getTitle());
		response.setDescription(entity.getDescription());
		response.setBusinessRelevance(entity.getBusinessRelevance());
		response.setIdeaCategoryId(entity.getIdeaCategoryId());
		response.setCompetenceCenter(entity.getCompetenceCenter());
		response.setIdeaType(entity.getIdeaType());
		response.setIdeaStatus(entity.getStatus());
		response.setPublished(entity.getPublished());
		response.setLocked(entity.getLocked());
		response.setBusinessDomain(entity.getBusinessDomain());
		response.setTechnologyDomain(entity.getTechnologyDomain());
		response.setTechStack(entity.getTechStack());
		response.setNovelty(entity.getNovelty());
		response.setClaims(entity.getClaims());
		response.setCompleteness(entity.getCompleteness());
		response.setL0ReviewerId(entity.getL0ReviewerId());
		response.setCreatedById(entity.getCreatedBy());
		response.setCreatedOn(entity.getCreatedDate());
		response.setDisclosed(entity.getDisclosed());
		response.setAccessLevel(entity.getAccessLevel());
		return response;
	}

	/**
	 * to retrieve level zero reviewer
	 * @param categoryId
	 * @return
	 * @throws InnovationServiceException
	 */
	private String getL0Reviwer(String categoryId) throws InnovationServiceException {
		String reviewerId = null;
		Optional<PgCategoryEntity> entity = pgIdeaCategoryRepository.findById(categoryId);
		if (entity.isPresent()) {
			PgCategoryEntity category = entity.get();
			List<PgReviewersEntity> reviewers = category.getReviewers().stream().filter(p -> p.getLevel().equals(ReviewLevelEnum.LEVEL_0)).collect(Collectors.toList());
			int index = new Random().nextInt(reviewers.size());
			reviewerId = reviewers.get(index).getUserId();
		}
		else
			throw new InnovationServiceException("category.not.found", "Category Not Found with id : " + categoryId);
		return reviewerId;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.altimetrik.playground.innovation.service.PgIdeaService#updateIdea(
	 * java.lang.String, com.altimetrik.playground.innovation.bean.PgIdeaRequest)
	 */
	@Override
	public PgIdeaResponse updateIdea(String ideaId, PgIdeaRequest pgIdeaRequest) throws PgApplicationException {
		String loggedInUser = userUtil.getLoggedInUser();
		PgIdeaEntity idea = getIdeaEntity(ideaId);
		if (access(idea.getTeam()) && !idea.getStatus().equals(IdeaStatusEnum.APPROVED)) {
			if (pgIdeaRequest.getStatus() != null) {
				if (pgIdeaRequest.getStatus().equals(IdeaStatusEnum.SUBMITTED)) {
					if (!idea.getPublished())
						throw new InnovationServiceException("not.published", "Idea Not Published for Submission for ID " + ideaId);
					if (idea.getIdeaCategoryId() != null)
						throw new InnovationServiceException(NOT_FOUND,
								IDEA_WITH_ID + ideaId + ") is not a part of any category for ID. Please select a category before Submission.");
					if (StringUtils.isNotBlank(idea.getNovelty()))
						throw new InnovationServiceException(NOT_FOUND, IDEA_WITH_ID + ideaId + ") does not have any novelty.");
					if (StringUtils.isNotBlank(idea.getClaims()))
						throw new InnovationServiceException(NOT_FOUND, IDEA_WITH_ID + ideaId + ") does not have any Claims.");
					if (StringUtils.isNotBlank(idea.getCompleteness()))
						throw new InnovationServiceException(NOT_FOUND, IDEA_WITH_ID + ideaId + ") does not have any Completeness.");
				}
				idea.setStatus(pgIdeaRequest.getStatus());
			}
			if (pgIdeaRequest.getTitle() != null) {
				if (StringUtils.isNotBlank(pgIdeaRequest.getTitle())) {
					if (!pgIdeaRequest.getTitle().equals(idea.getTitle())) {
						Optional<PgIdeaEntity> findByTitle = pgIdeaRepository.findByTitle(pgIdeaRequest.getTitle());
						if (findByTitle.isPresent())
							throw new InnovationServiceException("idea.exist", "Idea with given title already exist.");
						idea.setTitle(pgIdeaRequest.getTitle());
					}
				}
				else {
					throw new InnovationServiceException("not.blank", "Idea Title cannot be blank.");
				}
			}
			if (StringUtils.isNotBlank(pgIdeaRequest.getDescription()))
				idea.setDescription(pgIdeaRequest.getDescription());
			if (StringUtils.isNotBlank(pgIdeaRequest.getBusinessRelevance()))
				idea.setBusinessRelevance(pgIdeaRequest.getBusinessRelevance());
			if (StringUtils.isNotBlank(pgIdeaRequest.getIdeaCategoryId())) {
				PgCategoryEntity category = pgIdeaCategoryRepository.findById(pgIdeaRequest.getIdeaCategoryId())
						.orElseThrow(() -> new InnovationServiceException(NOT_FOUND, "Please provide a valid category."));
				idea.setIdeaCategoryId(category.getId());
				idea.setL0ReviewerId(getL0Reviwer(pgIdeaRequest.getIdeaCategoryId()));
			}
			if (pgIdeaRequest.getCompetenceCenter() != null)
				idea.setCompetenceCenter(pgIdeaRequest.getCompetenceCenter());
			if (pgIdeaRequest.getIdeaType() != null)
				idea.setIdeaType(pgIdeaRequest.getIdeaType());
			if (pgIdeaRequest.getPublished() != null && idea.getStatus().equals(IdeaStatusEnum.NOT_SUBMITTED)) {
				if (!pgIdeaRequest.getPublished() && idea.getPublished())
					throw new InnovationServiceException("restricted", "Idea once published cannot be made draft.");
				else
					idea.setPublished(pgIdeaRequest.getPublished());
			}
			if (pgIdeaRequest.getLocked() != null)
				idea.setLocked(pgIdeaRequest.getLocked());
			if (StringUtils.isNotBlank(pgIdeaRequest.getBusinessDomain()))
				idea.setBusinessDomain(pgIdeaRequest.getBusinessDomain());
			if (StringUtils.isNotBlank(pgIdeaRequest.getTechnologyDomain()))
				idea.setTechnologyDomain(pgIdeaRequest.getTechnologyDomain());
			if (StringUtils.isNotBlank(pgIdeaRequest.getTechStack()))
				idea.setTechStack(pgIdeaRequest.getTechStack());
			if (StringUtils.isNotBlank(pgIdeaRequest.getNovelty()))
				idea.setNovelty(pgIdeaRequest.getNovelty());
			if (StringUtils.isNotBlank(pgIdeaRequest.getClaims()))
				idea.setClaims(pgIdeaRequest.getClaims());
			if (StringUtils.isNotBlank(pgIdeaRequest.getCompleteness()))
				idea.setCompleteness(pgIdeaRequest.getCompleteness());
			if (StringUtils.isNotBlank(pgIdeaRequest.getIdeaCategoryId()))
				idea.setL0ReviewerId(getL0Reviwer(pgIdeaRequest.getIdeaCategoryId()));
			if (pgIdeaRequest.getDisclosed() != null)
				idea.setDisclosed(pgIdeaRequest.getDisclosed());
			if (StringUtils.isNotBlank(pgIdeaRequest.getAccessLevel()))
				idea.setAccessLevel(pgIdeaRequest.getAccessLevel());
			idea.setLastModifiedBy(loggedInUser);
			idea.setLastModifiedDate(new Date());
			idea = pgIdeaRepository.save(idea);
		}
		else {
			throw new InnovationServiceException(USER_NOT_AUTHORIZED, "Current user not authorized to update the idea.");
		}
		PgIdeaResponse pgIdeaResponse = prepareIdeaResponse(idea);
		pgIdeaResponse.setStatus(new StatusDetails(StatusDetails.SUCCESS));
		return pgIdeaResponse;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.altimetrik.playground.innovation.service.PgIdeaService#getTeamList(
	 * java.lang.String)
	 */
	@Override
	public PgIdeaTeamListResponse getTeamList(String ideaId) throws PgApplicationException {
		Optional<PgIdeaEntity> entity = pgIdeaRepository.findById(ideaId);
		if (!entity.isPresent()) {
			throw new InnovationServiceException(NOT_FOUND, "Idea Not Found for ID " + ideaId);
		}
		PgIdeaEntity ideaEntity = entity.get();
		PgIdeaTeamListResponse resp = new PgIdeaTeamListResponse();
		final List<PgTeamEntity> teamEntityList = ideaEntity.getTeam();
		AtomicBoolean isLoggedInUserPartOfIdea = new AtomicBoolean(false);
		List<PgIdeaTeamResponse> result = new ArrayList<>();
		if (teamEntityList != null) {
			teamEntityList.forEach(teamEntity -> {
				PgIdeaTeamResponse team = prepareTeamResponse(teamEntity);
				if (team.getUserId().equals(userUtil.getLoggedInUser()))
					isLoggedInUserPartOfIdea.set(true);
				result.add(team);
			});
			if (isLoggedInUserPartOfIdea.get()) {
				resp.setResult(result);
			}
			else if (ideaEntity.getPublished() != null && ideaEntity.getPublished()) {
				resp.setResult(result.stream().filter(p -> p.getTeamMemberRequestStatus() != null && p.getTeamMemberRequestStatus().equals(TeamMemberRequestStatusEnum.ACCEPTED))
						.collect(Collectors.toList()));
			}
		}
		resp.setStatus(new StatusDetails(StatusDetails.SUCCESS));
		return resp;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.altimetrik.playground.innovation.service.PgIdeaService#
	 * listTeamRequests(java.lang.String)
	 */
	@Override
	public PgIdeaTeamListResponse listTeamRequests(String ideaId) throws PgApplicationException {
		PgIdeaEntity ideaEntity = getIdeaEntity(ideaId);
		validateUserAccessToIdea(ideaEntity);
		PgIdeaTeamListResponse resp = new PgIdeaTeamListResponse();
		final List<PgTeamEntity> teamEntityList = ideaEntity.getTeam();
		List<PgIdeaTeamResponse> result = new ArrayList<>();
		if (teamEntityList != null) {
			teamEntityList.forEach(teamEntity -> {
				if (teamEntity.getRequestMode() != null
						&& (teamEntity.getRequestMode().equals(TeamMemberRequestModeEnum.INVITE) || teamEntity.getRequestMode().equals(TeamMemberRequestModeEnum.JOIN))) {
					PgIdeaTeamResponse team = prepareTeamResponse(teamEntity);
					result.add(team);
				}
			});
		}
		resp.setResult(result);
		resp.setStatus(new StatusDetails(StatusDetails.SUCCESS));
		return resp;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.altimetrik.playground.innovation.service.PgIdeaService#
	 * inviteTeamRequests(java.lang.String,
	 * com.altimetrik.playground.innovation.bean.PgIdeaTeamInviteRequest)
	 */
	@Override
	public BaseResponse inviteTeamRequests(String ideaId, PgIdeaTeamInviteRequest request) throws PgApplicationException {
		PgIdeaEntity ideaEntity = getIdeaEntity(ideaId);
		validateUserAccessToIdea(ideaEntity);
		userUtil.isUserAnEmployee(request.getUserId());
		BaseResponse res = new BaseResponse();
		List<PgTeamEntity> teamEntityList = ideaEntity.getTeam();
		if (teamEntityList != null) {
			for (PgTeamEntity teamEntity : teamEntityList) {
				if (teamEntity.getUserId().equals(request.getUserId())) {
					throw new InnovationServiceException("user.exists", request.getUserId() + " is already invited/part of team for this Idea.");
				}
			}
			PgTeamEntity entity = new PgTeamEntity();
			entity.setRole(RoleTypeEnum.valueOf(request.getRole()));
			entity.setUserId(request.getUserId());
			entity.setRequestMode(TeamMemberRequestModeEnum.INVITE);
			entity.setRequestStatus(TeamMemberRequestStatusEnum.PENDING);
			entity.setCreatedBy(userUtil.getLoggedInUser());
			entity.setCreatedDate(new Date());
			teamEntityList.add(entity);
		}
		ideaEntity.setTeam(teamEntityList);
		pgIdeaRepository.save(ideaEntity);
		res.setStatus(new StatusDetails(StatusDetails.SUCCESS));
		return res;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.altimetrik.playground.innovation.service.PgIdeaService#
	 * rejectTeamRequest(java.lang.String, java.lang.String)
	 */
	@Override
	public BaseResponse rejectTeamRequest(String ideaId, String requestId) throws PgApplicationException {
		return teamRequestAction(ideaId, requestId, TeamMemberRequestStatusEnum.REJECTED);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.altimetrik.playground.innovation.service.PgIdeaService#
	 * acceptTeamRequest(java.lang.String, java.lang.String)
	 */
	@Override
	public BaseResponse acceptTeamRequest(String ideaId, String requestId) throws PgApplicationException {
		return teamRequestAction(ideaId, requestId, TeamMemberRequestStatusEnum.ACCEPTED);
	}

	/**
	 * @param ideaId
	 * @param requestId
	 * @param status
	 * @return
	 * @throws InnovationServiceException
	 */
	private BaseResponse teamRequestAction(String ideaId, String requestId, TeamMemberRequestStatusEnum status) throws InnovationServiceException {
		PgIdeaEntity ideaEntity = getIdeaEntity(ideaId);
		BaseResponse res = new BaseResponse();
		List<PgTeamEntity> teamEntityList = ideaEntity.getTeam();
		if (teamEntityList != null) {
			PgTeamEntity teamEntity = teamEntityList.stream().filter(p -> p.getId().equals(requestId)).findAny()
					.orElseThrow(() -> new InnovationServiceException("request.not.found", "Request ID not found."));
			if (teamEntity.getUserId().equals(userUtil.getLoggedInUser())) {
				if (!teamEntity.getRequestStatus().equals(TeamMemberRequestStatusEnum.PENDING)) {
					throw new InnovationServiceException("invalid.state", "Team Request with PENDING status can only be updated.");
				}
				teamEntity.setRequestStatus(status);
				teamEntity.setLastModifiedBy(userUtil.getLoggedInUser());
				teamEntity.setLastModifiedDate(new Date());
			}
			else {
				throw new InnovationServiceException(USER_NOT_AUTHORIZED, "Current user not authorized to perform this action on this team request.");
			}
		}
		ideaEntity.setTeam(teamEntityList);
		pgIdeaRepository.save(ideaEntity);
		res.setStatus(new StatusDetails(StatusDetails.SUCCESS));
		return res;
	}

	/**
	 * @param ideaId
	 * @return
	 * @throws InnovationServiceException
	 */
	private PgIdeaEntity getIdeaEntity(String ideaId) throws InnovationServiceException {
		Optional<PgIdeaEntity> entity = pgIdeaRepository.findById(ideaId);
		if (!entity.isPresent()) {
			throw new InnovationServiceException(NOT_FOUND, "Idea Not Found for ID " + ideaId);
		}
		return entity.get();
	}

	/**
	 * @param teamEntity
	 * @return
	 */
	private PgIdeaTeamResponse prepareTeamResponse(PgTeamEntity teamEntity) {
		PgIdeaTeamResponse team = new PgIdeaTeamResponse();
		team.setId(teamEntity.getId());
		team.setMode(teamEntity.getRequestMode());
		team.setRole(teamEntity.getRole());
		team.setTeamMemberRequestStatus(teamEntity.getRequestStatus());
		team.setUserId(teamEntity.getUserId());
		team.setCreatedBy(teamEntity.getCreatedBy());
		team.setCreatedDate(teamEntity.getCreatedDate());
		team.setLastModifiedBy(teamEntity.getLastModifiedBy());
		return team;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.altimetrik.playground.innovation.service.PgIdeaService#joinRequest(
	 * java.lang.String)
	 */
	@Override
	public BaseResponse joinRequest(String ideaId) throws PgApplicationException {
		BaseResponse response = new BaseResponse();
		PgIdeaEntity idea = getIdeaEntity(ideaId);
		isIdeaLocked(idea);
		String loggedInUser = userUtil.getLoggedInUser();
		if (idea.getPublished() && idea.getIdeaType().equals(IdeaTypeEnum.PUBLIC)) {
			List<PgTeamEntity> teamEntityList = idea.getTeam();
			if (teamEntityList != null && teamEntityList.stream().anyMatch(teamEntity -> teamEntity.getUserId().equals(loggedInUser)))
				throw new InnovationServiceException("user.exists", loggedInUser + " is already invited/joined the team for this Idea.");
			PgTeamEntity member = new PgTeamEntity();
			member.setUserId(loggedInUser);
			member.setRole(RoleTypeEnum.MEMBER);
			member.setRequestMode(TeamMemberRequestModeEnum.JOIN);
			member.setRequestStatus(TeamMemberRequestStatusEnum.PENDING);
			member.setCreatedBy(loggedInUser);
			member.setCreatedDate(new Date());
			member.setLastModifiedBy(loggedInUser);
			idea.getTeam().add(member);
			pgIdeaRepository.save(idea);
		}
		else {
			throw new InnovationServiceException(USER_NOT_AUTHORIZED, "Cannot join this idea(" + ideaId + "), as it is either not published or is PRIVATE");
		}
		response.setStatus(new StatusDetails(StatusDetails.SUCCESS));
		return response;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.altimetrik.playground.innovation.service.PgIdeaService#
	 * acceptJoinRequests(java.lang.String, java.lang.String)
	 */
	@Override
	public BaseResponse acceptJoinRequests(String ideaId, String requestId) throws PgApplicationException {
		return handleJoinRequests(TeamMemberRequestStatusEnum.ACCEPTED, ideaId, requestId);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.altimetrik.playground.innovation.service.PgIdeaService#
	 * rejectJoinRequests(java.lang.String, java.lang.String)
	 */
	@Override
	public BaseResponse rejectJoinRequests(String ideaId, String requestId) throws PgApplicationException {
		return handleJoinRequests(TeamMemberRequestStatusEnum.REJECTED, ideaId, requestId);
	}

	/**
	 * @param status
	 * @param ideaId
	 * @param requestId
	 * @return
	 * @throws PgApplicationException
	 */
	private BaseResponse handleJoinRequests(TeamMemberRequestStatusEnum status, String ideaId, String requestId) throws PgApplicationException {
		String loggedInUser = userUtil.getLoggedInUser();
		PgIdeaEntity ideaEntity = getIdeaEntity(ideaId);
		BaseResponse res = new BaseResponse();
		List<PgTeamEntity> teamEntityList = ideaEntity.getTeam();
		if (access(teamEntityList)) {
			PgTeamEntity teamEntity = teamEntityList.stream().filter(p -> p.getId().equals(requestId)).findAny()
					.orElseThrow(() -> new InnovationServiceException("request.not.found", "Request ID not found."));
			int index = teamEntityList.indexOf(teamEntity);
			teamEntity.setRequestStatus(status);
			teamEntity.setLastModifiedBy(loggedInUser);
			teamEntity.setLastModifiedDate(new Date());
			teamEntityList.set(index, teamEntity);
		}
		else {
			throw new InnovationServiceException(USER_NOT_AUTHORIZED, "Current user not authorized to perform this action on this team request.");
		}
		ideaEntity.setTeam(teamEntityList);
		pgIdeaRepository.save(ideaEntity);
		res.setStatus(new StatusDetails(StatusDetails.SUCCESS));
		return res;
	}

	/**
	 * @param teamEntityList
	 * @return
	 * @throws InnovationServiceException
	 */
	private Boolean access(List<PgTeamEntity> teamEntityList) throws InnovationServiceException {
		PgTeamEntity admin = teamEntityList.stream().filter(p -> p.getUserId().equals(userUtil.getLoggedInUser())).findAny()
				.orElseThrow(() -> new InnovationServiceException(USER_NOT_AUTHORIZED, USER_NOT_AUTHORIZED_MSG));
		return admin.getRole().equals(RoleTypeEnum.INNOVATOR) || admin.getRole().equals(RoleTypeEnum.CO_INNOVATOR);
	}

	/**
	 * @param ideaEntity
	 * @throws InnovationServiceException
	 */
	private void validateUserAccessToIdea(PgIdeaEntity ideaEntity) throws InnovationServiceException {
		List<PgTeamEntity> teamEntityList = ideaEntity.getTeam();
		if (teamEntityList != null)
			teamEntityList.stream().filter(
					p -> p.getUserId().equals(userUtil.getLoggedInUser()) && (p.getRequestStatus() == null || p.getRequestStatus().equals(TeamMemberRequestStatusEnum.ACCEPTED)))
					.findAny().orElseThrow(() -> new InnovationServiceException(USER_NOT_AUTHORIZED, USER_NOT_AUTHORIZED_MSG));
		else
			throw new InnovationServiceException(USER_NOT_AUTHORIZED, USER_NOT_AUTHORIZED_MSG);
	}

	private void isIdeaLocked(PgIdeaEntity ideaEntity) throws InnovationServiceException {
		if (ideaEntity.getLocked())
			throw new InnovationServiceException("idea.locked", "Idea Locked.");
	}
}
