/*******************************************************************************
 * Copyright (C) Altimetrik 2018. All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Altimetrik. You shall not disclose such Confidential Information
 * and shall use it only in accordance with the terms and conditions
 * entered into with Altimetrik.
 ******************************************************************************/
package com.altimetrik.playground.innovation.util;

import java.util.List;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Service;

import com.altimetrik.playground.innovation.entity.BaseEntity;
import com.mongodb.Function;

@Service
public class MongoUtils {

	@Autowired
	private MongoTemplate mongo;

	public <D, N extends BaseEntity> N findNestedDocument(Class<D> docClass, String collectionName, String outerId, String innerId, Function<D, List<N>> collectionGetter) {
		// get index of subdocument in array
		Query query = new Query(Criteria.where("_id").is(outerId).and(collectionName + "._id").is(innerId));
		query.fields().include(collectionName + "._id");
		D obj = mongo.findOne(query, docClass);
		if (obj == null) {
			return null;
		}
		List<String> itemIds = collectionGetter.apply(obj).stream().map(N::getId).collect(Collectors.toList());
		int index = itemIds.indexOf(innerId);
		if (index == -1) {
			return null;
		}

		// retrieve subdocument at index using slice operator
		Query query2 = new Query(Criteria.where("_id").is(outerId).and(collectionName + "._id").is(innerId));
		query2.fields().include(collectionName).slice(collectionName, index, 1);
		D obj2 = mongo.findOne(query2, docClass);
		if (obj2 == null) {
			return null;
		}
		return collectionGetter.apply(obj2).get(0);
	}

	public void removeNestedDocument(String outerId, String innerId, String collectionName, Class<?> outerClass) {
		Update update = new Update();
		update.pull(collectionName, new Query(Criteria.where("_id").is(innerId)));
		mongo.updateFirst(new Query(Criteria.where("_id").is(outerId)), update, outerClass);
	}
}
