package com.altimetrik.playground.innovation.util;

import java.util.concurrent.atomic.AtomicBoolean;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import com.altimetrik.playground.innovation.bean.Principal;
import com.altimetrik.playground.innovation.bean.usermanagement.UserResponseBean;
import com.altimetrik.playground.innovation.exception.InnovationServiceException;

import lombok.extern.slf4j.Slf4j;

/**
 * The Class UserUtil.
 */
@Component("userUtil")
@Slf4j
public class UserUtil {
	@Value("${playground.serviceUrl}")
	private String baseUrl;

	/**
	 * Gets the logged in user.
	 *
	 * @return the logged in user
	 */
	public String getLoggedInUser() {
		Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
		Principal principal = (Principal) authentication.getPrincipal();
		return principal.getUserId();
	}

	public UserResponseBean getUserDetailsByEmail(String email) throws InnovationServiceException {
		try {
			String umCall = baseUrl + "api/common/getDetailsByEmail/" + email + "/";
			log.info("FINAL URL : " + umCall);
			RestTemplate restTemplate = new RestTemplate();
			HttpHeaders httpHeaders = new HttpHeaders();
			httpHeaders.setContentType(MediaType.APPLICATION_JSON);
			HttpEntity<String> httpEntity = new HttpEntity<>(httpHeaders);
			ResponseEntity<UserResponseBean> obj = restTemplate.exchange(umCall, HttpMethod.GET, httpEntity, UserResponseBean.class);
			UserResponseBean body = obj.getBody();
			if (body.getInfoDetails() == null)
				throw new InnovationServiceException("user.not.found", "User details not found in userManagement service for email " + email);
			return body;
		}
		catch (Exception e) {
			log.error("ERROR from user mgmt :: ", e);
			throw new InnovationServiceException("user.service", "Exception in getting user details for email " + email);
		}
	}

	public void isUserAnEmployee(String userId) throws InnovationServiceException {
		UserResponseBean userDetailsByEmail = getUserDetailsByEmail(userId);
		AtomicBoolean isEmployee = new AtomicBoolean(false);
		userDetailsByEmail.getInfoDetails().getPgUserInfoMstr().getPgRoleUser().stream().forEach(role -> {
			if (role.getPgRolePrivilege() != null && role.getPgRolePrivilege().getPgRoleMstr() != null
					&& "EMPLOYEE".equalsIgnoreCase(role.getPgRolePrivilege().getPgRoleMstr().getRoleName())) {
				isEmployee.set(true);
			}
		});
		if (!isEmployee.get())
			throw new InnovationServiceException("not.employee", "Team member is not an employee.");
	}
}
