/*******************************************************************************
 * Copyright (C) Altimetrik 2018. All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Altimetrik. You shall not disclose such Confidential Information
 * and shall use it only in accordance with the terms and conditions
 * entered into with Altimetrik.
 ******************************************************************************/
/*package com.altimetrik.playground.innovation.service.impl;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.mockito.Spy;
import org.skyscreamer.jsonassert.JSONAssert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;

import com.altimetrik.playground.innovation.bean.PgIdeaCategoryResponseBean;
import com.altimetrik.playground.innovation.bean.PgIdeaCollateralRequestBean;
import com.altimetrik.playground.innovation.bean.PgIdeaCollateralResponseBean;
import com.altimetrik.playground.innovation.bean.PgIdeaRequestBean;
import com.altimetrik.playground.innovation.bean.PgIdeaResponseBean;
import com.altimetrik.playground.innovation.bean.PgIdeaStatusRequestBean;
import com.altimetrik.playground.innovation.bean.PgUserRequestBean;
import com.altimetrik.playground.innovation.bean.PgUserResponseBean;
import com.altimetrik.playground.innovation.constants.ArtifactModeEnum;
import com.altimetrik.playground.innovation.constants.ArtifactTypeEnum;
import com.altimetrik.playground.innovation.constants.IdeaStatusEnum;
import com.altimetrik.playground.innovation.constants.RoleTypeEnum;
import com.altimetrik.playground.innovation.constants.UserTypeEnum;
import com.altimetrik.playground.innovation.entity.PgIdeaCategoryEntity;
import com.altimetrik.playground.innovation.entity.PgIdeaArtifactEntity;
import com.altimetrik.playground.innovation.entity.PgIdeaEntity;
import com.altimetrik.playground.innovation.entity.PgIdeaReviewerEntity;
import com.altimetrik.playground.innovation.entity.PgIdeaReviewerId;
import com.altimetrik.playground.innovation.entity.PgIdeaTeamMemberEntity;
import com.altimetrik.playground.innovation.entity.PgIdeaTeamMemberId;
import com.altimetrik.playground.innovation.entity.PgUserEntity;
import com.altimetrik.playground.innovation.exception.AuthorizationException;
import com.altimetrik.playground.innovation.exception.InnovationServiceException;
import com.altimetrik.playground.innovation.exception.InvalidStatusTransitionException;
import com.altimetrik.playground.innovation.properties.ErrorProperties;
import com.altimetrik.playground.innovation.repository.PgIdeaCategoryRepository;
import com.altimetrik.playground.innovation.repository.PgIdeaRepository;
import com.google.gson.Gson;

@SpringBootTest
@RunWith(SpringRunner.class)
public class PgIdeaServiceImplTests {

	@Autowired
	private Gson gson;

	@Spy
	private ErrorProperties errorProperties;

	@InjectMocks
	private PgIdeaServiceImpl pgIdeaService;

	@Mock
	private PgIdeaRepository pgIdeaRepository;

	@Mock
	private PgIdeaCategoryRepository pgIdeaCategoryRepository;

	@Before
	public void setUp() {
		MockitoAnnotations.initMocks(this);
	}

	private PgIdeaEntity mockPgIdea() {
		return new PgIdeaEntity("innovator@mail.com", Long.valueOf(1), "Idea", "Idea", "Idea", true, true, IdeaStatusEnum.DRAFTED, "reviewer@mail.com", this.mockPgIdeaCategory(),
				this.mockPgIdeaTeamMember(RoleTypeEnum.INNOVATOR), this.mockPgIdeaCollateral());
	}

	private PgIdeaCategoryEntity mockPgIdeaCategory() {
		final Set<PgIdeaReviewerEntity> pgIdeaReviewers = new HashSet<PgIdeaReviewerEntity>();
		pgIdeaReviewers.add(this.mockPgIdeaReviewer());
		return new PgIdeaCategoryEntity("admin@mail.com", Long.valueOf(1), "Category", pgIdeaReviewers);
	}

	private Set<PgIdeaTeamMemberEntity> mockPgIdeaTeamMember(RoleTypeEnum roleType) {
		final Set<PgIdeaTeamMemberEntity> pgIdeaTeamMembers = new HashSet<PgIdeaTeamMemberEntity>();
		pgIdeaTeamMembers.add(new PgIdeaTeamMemberEntity("innovator@mail.com", new PgIdeaTeamMemberId(Long.valueOf(1), Long.valueOf(1)), RoleTypeEnum.INNOVATOR,
				this.mockPgUser(RoleTypeEnum.INNOVATOR)));
		return pgIdeaTeamMembers;
	}

	private Set<PgIdeaArtifactEntity> mockPgIdeaCollateral() {
		final Set<PgIdeaArtifactEntity> pgIdeaCollaterals = new HashSet<PgIdeaArtifactEntity>();
		pgIdeaCollaterals.add(new PgIdeaArtifactEntity("innovator@mail.com", Long.valueOf(1), "link", ArtifactTypeEnum.LINK, ArtifactModeEnum.IDEA));
		return pgIdeaCollaterals;
	}

	private PgIdeaReviewerEntity mockPgIdeaReviewer() {
		return new PgIdeaReviewerEntity("admin@mail.com", new PgIdeaReviewerId(Long.valueOf(1), Long.valueOf(1)), this.mockPgUser(RoleTypeEnum.REVIEWER));
	}

	private PgUserEntity mockPgUser(RoleTypeEnum roleType) {
		PgUserEntity pgUser = null;
		switch (roleType) {
		case REVIEWER:
			pgUser = new PgUserEntity("admin@mail.com", Long.valueOf(1), "Reviewer", "reviewer@mail.com");
			break;
		case INNOVATOR:
			pgUser = new PgUserEntity("innovator@mail.com", Long.valueOf(1), "Innovator", "innovator@mail.com");
			break;
		}
		return pgUser;
	}

	private PgIdeaRequestBean mockPgIdeaRequest() {
		return new PgIdeaRequestBean("Idea", true, "Idea", true, "Idea", Long.valueOf(1), IdeaStatusEnum.DRAFTED, this.mockPgIdeaTeamMemberRequest(RoleTypeEnum.INNOVATOR),
				this.mockPgIdeaCollateralRequest(), "admin@mail.com");
	}

	private Set<PgUserRequestBean> mockPgIdeaTeamMemberRequest(RoleTypeEnum roleType) {
		final Set<PgUserRequestBean> pgIdeaTeamMembers = new HashSet<PgUserRequestBean>();
		pgIdeaTeamMembers.add(this.mockPgUserRequest(roleType));
		return pgIdeaTeamMembers;
	}

	private Set<PgIdeaCollateralRequestBean> mockPgIdeaCollateralRequest() {
		final Set<PgIdeaCollateralRequestBean> pgIdeaCollaterals = new HashSet<PgIdeaCollateralRequestBean>();
		pgIdeaCollaterals.add(new PgIdeaCollateralRequestBean("link", ArtifactTypeEnum.LINK, ArtifactModeEnum.IDEA));
		return pgIdeaCollaterals;
	}

	private PgUserRequestBean mockPgUserRequest(RoleTypeEnum roleType) {
		PgUserRequestBean pgUserRequest = null;
		switch (roleType) {
		case INNOVATOR:
			pgUserRequest = new PgUserRequestBean("innovator@mail.com", "Innovator", "innovator@mail.com", Long.valueOf(1), roleType);
			break;
		}
		return pgUserRequest;
	}

	private List<PgIdeaResponseBean> mockPgIdeaResponse() {
		List<PgIdeaResponseBean> pgIdeaResponses = new ArrayList<PgIdeaResponseBean>();
		pgIdeaResponses.add(new PgIdeaResponseBean(Long.valueOf(1), "Idea", true, "Idea", true, "Idea", IdeaStatusEnum.DRAFTED, this.mockPgIdeaCategoryResponse(),
				this.mockPgIdeaTeamMemberResponse(RoleTypeEnum.INNOVATOR), this.mockPgIdeaCollateralResponse()));
		return pgIdeaResponses;
	}

	private PgIdeaCategoryResponseBean mockPgIdeaCategoryResponse() {
		final Set<PgUserResponseBean> pgIdeaReviewerResponses = new HashSet<PgUserResponseBean>();
		pgIdeaReviewerResponses.add(this.mockPgUserResponse(RoleTypeEnum.REVIEWER));
		return new PgIdeaCategoryResponseBean(1, "Category", pgIdeaReviewerResponses);
	}

	private Set<PgUserResponseBean> mockPgIdeaTeamMemberResponse(RoleTypeEnum roleType) {
		final Set<PgUserResponseBean> pgIdeaTeamMemberResponses = new HashSet<PgUserResponseBean>();
		pgIdeaTeamMemberResponses.add(this.mockPgUserResponse(roleType));
		return pgIdeaTeamMemberResponses;
	}

	private Set<PgIdeaCollateralResponseBean> mockPgIdeaCollateralResponse() {
		final Set<PgIdeaCollateralResponseBean> pgIdeaCollateralResponses = new HashSet<PgIdeaCollateralResponseBean>();
		pgIdeaCollateralResponses.add(new PgIdeaCollateralResponseBean(Long.valueOf(1), "link", ArtifactTypeEnum.LINK, ArtifactModeEnum.IDEA));
		return pgIdeaCollateralResponses;
	}

	private PgUserResponseBean mockPgUserResponse(RoleTypeEnum roleType) {
		PgUserResponseBean pgUserResponse = null;
		switch (roleType) {
		case REVIEWER:
			pgUserResponse = new PgUserResponseBean("Reviewer", "reviewer@mail.com", Long.valueOf(1), roleType);
			break;
		case INNOVATOR:
			pgUserResponse = new PgUserResponseBean("Innovator", "innovator@mail.com", Long.valueOf(1), roleType);
			break;
		}
		return pgUserResponse;
	}

	@Test
	public void createIdeaTest() throws Exception {
		Mockito.when(this.pgIdeaRepository.findByTitle(Mockito.any())).thenReturn(null);
		Mockito.when(this.pgIdeaCategoryRepository.findByIdeaCategoryId(Mockito.any())).thenReturn(this.mockPgIdeaCategory());
		Mockito.when(pgIdeaRepository.save(Mockito.any())).thenReturn(this.mockPgIdea());
		JSONAssert.assertEquals(String.valueOf(true), String.valueOf(this.pgIdeaService.saveIdea(this.mockPgIdeaRequest())), true);
	}

	@Test
	public void updateIdeaTest() throws Exception {
		Mockito.when(this.pgIdeaRepository.findByTitle(Mockito.any())).thenReturn(this.mockPgIdea());
		Mockito.when(this.pgIdeaCategoryRepository.findByIdeaCategoryId(Mockito.any())).thenReturn(this.mockPgIdeaCategory());
		Mockito.when(this.pgIdeaRepository.save(Mockito.any())).thenReturn(this.mockPgIdea());
		JSONAssert.assertEquals(String.valueOf(true), String.valueOf(this.pgIdeaService.saveIdea(this.mockPgIdeaRequest())), true);
	}

	@Test
	public void populateCollateralsTest() throws Exception {
		this.pgIdeaService.populateCollaterals(null, this.mockPgIdea(), "admin@mail.com");
	}

	@Test
	public void populateIdeaCollateralResponsesTest() throws Exception {
		this.pgIdeaService.populateIdeaCollateralResponses(null);
	}

	@Test
	public void findAllIdeasTest() throws Exception {
		final List<PgIdeaEntity> pgIdeas = new ArrayList<PgIdeaEntity>();
		pgIdeas.add(this.mockPgIdea());
		Mockito.when(this.pgIdeaRepository.findAll()).thenReturn(pgIdeas);
		JSONAssert.assertEquals(this.gson.toJson(this.mockPgIdeaResponse()), this.gson.toJson(this.pgIdeaService.findAllIdeas(IdeaStatusEnum.ALL)), true);
	}

	@Test
	public void findAllIdeasByIdeaStatusTest() throws Exception {
		final List<PgIdeaEntity> pgIdeas = new ArrayList<PgIdeaEntity>();
		pgIdeas.add(this.mockPgIdea());
		Mockito.when(this.pgIdeaRepository.findByIdeaStatus(IdeaStatusEnum.DRAFTED)).thenReturn(pgIdeas);
		JSONAssert.assertEquals(this.gson.toJson(this.mockPgIdeaResponse()), this.gson.toJson(this.pgIdeaService.findAllIdeas(IdeaStatusEnum.DRAFTED)), true);
	}

	@Test
	public void findAllIdeasNullCheckTest() throws Exception {
		Mockito.when(this.pgIdeaRepository.findAll()).thenReturn(null);
		JSONAssert.assertEquals(this.gson.toJson(new ArrayList<PgIdeaResponseBean>()), this.gson.toJson(this.pgIdeaService.findAllIdeas(IdeaStatusEnum.ALL)), true);
	}

	@Test
	public void findAllIdeasByUserTypeAndIdeaStatusTest() throws Exception {
		final List<PgIdeaEntity> pgIdeas = new ArrayList<PgIdeaEntity>();
		pgIdeas.add(this.mockPgIdea());
		Mockito.when(this.pgIdeaService.findAllUserIdeas(UserTypeEnum.TEAM_MEMBER, Long.valueOf(1), IdeaStatusEnum.ALL)).thenReturn(pgIdeas);
		JSONAssert.assertEquals(this.gson.toJson(this.mockPgIdeaResponse()),
				this.gson.toJson(this.pgIdeaService.findAllIdeas(UserTypeEnum.TEAM_MEMBER, Long.valueOf(1), IdeaStatusEnum.ALL)), true);
	}

	@Test
	public void findAllIdeasByUserTypeAndIdeaStatusNullCheckTest() throws Exception {
		Mockito.when(this.pgIdeaService.findAllUserIdeas(UserTypeEnum.TEAM_MEMBER, Long.valueOf(1), IdeaStatusEnum.ALL)).thenReturn(null);
		JSONAssert.assertEquals(this.gson.toJson(new ArrayList<PgIdeaResponseBean>()),
				this.gson.toJson(this.pgIdeaService.findAllIdeas(UserTypeEnum.TEAM_MEMBER, Long.valueOf(1), IdeaStatusEnum.ALL)), true);
	}

	@Test
	public void findAllReviewerIdeasTest() throws Exception {
		final List<PgIdeaEntity> pgIdeas = new ArrayList<PgIdeaEntity>();
		Mockito.when(this.pgIdeaRepository.findByIdeaReviewerId(Long.valueOf(1))).thenReturn(pgIdeas);
		JSONAssert.assertEquals(this.gson.toJson(pgIdeas), this.gson.toJson(this.pgIdeaService.findAllUserIdeas(UserTypeEnum.REVIEWER, Long.valueOf(1), IdeaStatusEnum.ALL)), true);
	}

	@Test
	public void findAllReviewerIdeasByIdeaStatusTest() throws Exception {
		final List<PgIdeaEntity> pgIdeas = new ArrayList<PgIdeaEntity>();
		Mockito.when(this.pgIdeaRepository.findByIdeaReviewerIdAndIdeaStatus(Long.valueOf(1), IdeaStatusEnum.SUBMITTED)).thenReturn(pgIdeas);
		JSONAssert.assertEquals(this.gson.toJson(pgIdeas), this.gson.toJson(this.pgIdeaService.findAllUserIdeas(UserTypeEnum.REVIEWER, Long.valueOf(1), IdeaStatusEnum.SUBMITTED)),
				true);
	}

	@Test
	public void findAllTeamMemberIdeasTest() throws Exception {
		final List<PgIdeaEntity> pgIdeas = new ArrayList<PgIdeaEntity>();
		Mockito.when(this.pgIdeaRepository.findByIdeaTeamMemberId(Long.valueOf(1))).thenReturn(pgIdeas);
		JSONAssert.assertEquals(this.gson.toJson(pgIdeas), this.gson.toJson(this.pgIdeaService.findAllUserIdeas(UserTypeEnum.TEAM_MEMBER, Long.valueOf(1), IdeaStatusEnum.ALL)),
				true);
	}

	@Test
	public void findAllTeamMemberIdeasByIdeaStatusTest() throws Exception {
		final List<PgIdeaEntity> pgIdeas = new ArrayList<PgIdeaEntity>();
		Mockito.when(this.pgIdeaRepository.findByIdeaTeamMemberIdAndIdeaStatus(Long.valueOf(1), IdeaStatusEnum.SUBMITTED)).thenReturn(pgIdeas);
		JSONAssert.assertEquals(this.gson.toJson(pgIdeas),
				this.gson.toJson(this.pgIdeaService.findAllUserIdeas(UserTypeEnum.TEAM_MEMBER, Long.valueOf(1), IdeaStatusEnum.SUBMITTED)), true);
	}

	@Test
	public void updateIdeaStatusTest() throws Exception {
		final PgIdeaStatusRequestBean pgIdeaStatusRequest = new PgIdeaStatusRequestBean(Long.valueOf(1), IdeaStatusEnum.SUBMITTED);
		Mockito.when(this.pgIdeaRepository.findByIdeaId(Long.valueOf(1))).thenReturn(this.mockPgIdea());
		Mockito.when(this.pgIdeaRepository.save(Mockito.any())).thenReturn(this.mockPgIdea());
		JSONAssert.assertEquals(String.valueOf(true), String.valueOf(this.pgIdeaService.updateIdeaStatus(pgIdeaStatusRequest)), true);
	}

	@Test(expected = InvalidStatusTransitionException.class)
	public void updateInvalidIdeaStatusTest() throws InvalidStatusTransitionException, InnovationServiceException, AuthorizationException {
		final PgIdeaEntity pgIdea = this.mockPgIdea();
		pgIdea.setIdeaStatus(IdeaStatusEnum.SUBMITTED);
		final PgIdeaStatusRequestBean pgIdeaStatusRequest = new PgIdeaStatusRequestBean(Long.valueOf(1), IdeaStatusEnum.DRAFTED);
		Mockito.when(this.pgIdeaRepository.findByIdeaId(Long.valueOf(1))).thenReturn(pgIdea);
		this.pgIdeaService.updateIdeaStatus(pgIdeaStatusRequest);
	}

}
*/
